import string
from numpy import * 
#zeros, arange, prod, ndim, array, float64, flipud, fix, size, mean, abs, shape, hstack
import scipy.optimize
import datetime
import dbg

def io_write(input_type,inputs,file_name,fmt,delimiter):
    """Write various data into the file 
    INPUT = 
        input_type : defines the output file
        inputs : variables and data that are written into the file
        file_name : defines the file to write in
        fmt : defines the format to store numbers
        delimiter : defines the delimiter between two adjacent numbers
    OUTPUT = 
    """
    file_obj = open(file_name,'w')
    
    if input_type == 'input_FFTH':
        file_obj.write('#FFTH input file' + '\n')
        file_obj.write('#file was created on ' + str(datetime.datetime.now()) + '\n')
        file_obj.write('#Y - the vector from \set{R}^2 storing the size of a periodic unit cell in each direction' + '\n')
        file_obj.write("#E - the matrix from \set{R}^{N_1xN_2} storing the Young's modulus" + '\n')
        file_obj.write("#nu - the matrix from \set{R}^{N_1xN_2} storing the Poisson's ratio" + '\n')
    elif input_type == 'output_FFTH':
        file_obj.write('#FFTH output file' + '\n')
        file_obj.write('#file was created on ' + str(datetime.datetime.now()) + '\n')
        file_obj.write("#" + '\n')
        file_obj.write("#basic info about engineering notation:" + '\n')
        file_obj.write("#the data are plotted in Mandel's notation, i.e. strain = [strain_11, strain_22, (2^.5)*strain_12];" + '\n')
        file_obj.write("#    and stress = [stress_11, stress_22, (2^.5)*stress_12]" + '\n')
        file_obj.write("#stiffness L = [[L_1111, L_1122, (2^.5)*L_1112]" + '\n')
        file_obj.write("#               [L_1122, L_2222, (2^.5)*L_2212]" + '\n')
        file_obj.write("#               [(2^.5)*L_1112, (2^.5)*L_1212, 2*L_2212]]" + '\n')
        file_obj.write("#" + '\n')
        file_obj.write("#OUTPUT VARIABLES:" + '\n')
        file_obj.write("#L_eff - homogenized matrix in Mandel's notation" + '\n')
        file_obj.write('#resnorm - residual based norm calculated for unit loads, i.e. sequentially for strain_0 = [1,0,0]; strain_0 = [0,1,0] and strain_0 = [0,0,1]' + '\n')
        file_obj.write('#kit - number of iteration of Conjugate gradients reached for unit loads' + '\n')
        file_obj.write('#maxit - maximal number of iteration for Conjugate gradients' + '\n')
        file_obj.write('#tol - required tolerance for residual based norm to reach a convergence of Conjugate gradients' + '\n')
        file_obj.write('#time - time in seconds to calculate Conjugate gradients for a particular unit load' + '\n')
        file_obj.write("#L - the object from \set{R}^{dxdxN_1xN_2} storing the stiffness coefficients" + '\n')
        file_obj.write("#strain - the object from \set{R}^{dxdxN_1xN_2} storing the strain from unit loads, strain[i][j] stores the strain component strain_j from i-th unit load (in engineering notation)" + '\n')
    elif input_type == 'output_analytical':
        file_obj.write('#output file of analytical homogenization' + '\n')
        file_obj.write('#file was created on ' + str(datetime.datetime.now()) + '\n')
        file_obj.write("#" + '\n')
        file_obj.write("#E_V,nu_V : Voight lower bound in terms of Young's modulus and Poison's ratio" + '\n')
        file_obj.write("#E_R,nu_R : Reuss upper bound in terms of Young's modulus and Poison's ratio" + '\n')
        file_obj.write("#E_MT,nu_MT : Mori-Tanaka homogenized Young's modulus and Poison's ratio" + '\n')
        file_obj.write("#E_SCS,nu_SCS : Self Consistent homogenized Young's modulus and Poison's ratio" + '\n')
        file_obj.write("#E_HSW_lower,nu_HSW_lower : Hashin-Shtrikman-Walpole lower bound in terms of Young's modulus and Poison's ratio" + '\n')
        file_obj.write("#E_HSW_upper,nu_HSW_upper : Hashin-Shtrikman-Walpole upper bound in terms of Young's modulus and Poison's ratio" + '\n')
    elif input_type == 'input_ve':
        file_obj.write('#Input for viscoelastic homogenization' + '\n')
        file_obj.write('#file was created on ' + str(datetime.datetime.now()) + '\n')
        file_obj.write("#" + '\n')
        file_obj.write("#N - the vector from \set{R}^2 storing the number of indentation points in each direction" + '\n')
        file_obj.write('#Y - the vector from \set{R}^2 storing the size of a periodic unit cell in each direction' + '\n')
        file_obj.write("#E - the matrix from \set{R}^{N_1xN_2} storing the Young's modulus" + '\n')
        file_obj.write("#nu - the matrix from \set{R}^{N_1xN_2} storing the Poisson's ratio" + '\n')
        file_obj.write("#eta_M - the matrix from \set{R}^{N_1xN_2} storing the viscoelastic parameter \eta_M" + '\n')
        file_obj.write("#G_V - the matrix from \set{R}^{N_1xN_2} storing the viscoelastic parameter G_V" + '\n')
        file_obj.write("#eta_V - the matrix from \set{R}^{N_1xN_2} storing the viscoelastic parameter \eta_V" + '\n')
        file_obj.write("#t_L - the matrix from \set{R}^{N_1xN_2} storing the duration of loading phase" + '\n')
        file_obj.write("#t_H - the matrix from \set{R}^{N_1xN_2} storing the duration of holding phase" + '\n')
    elif input_type == 'output_ve':
        file_obj.write('#Output of viscoelastic homogenization' + '\n')
        file_obj.write('#file was created on ' + str(datetime.datetime.now()) + '\n')
        file_obj.write("#" + '\n')
        file_obj.write("#L_eff - homogenized matrix in Mandel's notation" + '\n')
        file_obj.write('#G_eff - homogenized shear modulus, i.e. L[2][2]/2' + '\n')
        file_obj.write('#etaM_eff - homogenized \eta_M' + '\n')
        file_obj.write('#GV_eff - homogenized G_V' + '\n')
        file_obj.write('#etaV_eff - homogenized \eta_V' + '\n')
        file_obj.write('#strain33_mean - mean of shear strain at time discretization points' + '\n')
        file_obj.write('#strain33_holding_approx - the approximation of strain33_mean in holding phase with KVM model' + '\n')
        file_obj.write('#stress33_mean - mean of shear stress at time discretization points' + '\n')
        file_obj.write('#time - time discretization points' + '\n')
        file_obj.write('#time_holding - the end of holding phase time' + '\n')
        file_obj.write('#time_loading - the end of loading phase time' + '\n')
    else:
        file_obj.write('#file was created on ' + str(datetime.datetime.now()) + '\n')
        
    for var in inputs.keys():
        if ndim(inputs[var]) == 0:
            file_obj.write('\n' + var + '\n')
            file_obj.write(fmt % inputs[var] + '\n')
            file_obj.write('end' + '\n')
        elif ndim(inputs[var]) == 1:
            file_obj.write('\n' + var + '\n')
            ncol = inputs[var].shape[0]
            fmt_ncol = [fmt,]*ncol
            fmt_ncol = delimiter.join(fmt_ncol)
            file_obj.write(fmt_ncol % tuple(inputs[var]) + '\n')
            file_obj.write('end' + '\n')
        elif ndim(inputs[var]) == 2:
            file_obj.write('\n' + var + '\n')
            ncol = inputs[var].shape[1]
            fmt_ncol = [fmt,]*ncol
            fmt_ncol = delimiter.join(fmt_ncol)
            for row in inputs[var]:
                file_obj.write(fmt_ncol % tuple(row) + '\n')
            file_obj.write('end' + '\n')
        elif ndim(inputs[var]) == 4:
            D = inputs[var].shape[0]
            ncol = inputs[var].shape[3]
            fmt_ncol = [fmt,]*ncol
            fmt_ncol = delimiter.join(fmt_ncol)
            for i in arange(D):
                for j in arange(D):
                    file_obj.write('\n' + var + '[' + str(i) + ']' + '[' + str(j) + ']'  + '\n')
                    matrix = inputs[var][i][j]
                    for row in matrix:
                        file_obj.write(fmt_ncol % tuple(row) + '\n')
                    file_obj.write('end' + '\n')
        else:
            print('The variable "' + var + '" cannot be written down.')
        
    file_obj.close()
    return

def io_read(file_name,variables):
    """Read data from input file made directly for both elasticity or visco-elasticity FFT homogenization  
    INPUT =
        file_name : the name of file to read from
        variables : the list of variables that are intended to read 
    OUTPUT =
        output : dictionary containing the read data, with the components of variables as keys
    """
    comments = ['#','%']
    output = dict()
    file_obj = open(file_name,'r')
    line = file_obj.readline()
    while line != "":
        if line.strip() is None or line.strip() == '' or line[0] in comments:
            line = file_obj.readline()
            continue
        elif line.strip() in variables or line.strip()[0:6] == 'strain' or line.strip()[0] == 'L':
            variable = line.strip()
            line = file_obj.readline()
            variable_data = []
            while line.strip() != 'end':
                variable_data.append(line.split())
                line = file_obj.readline()
            output[variable] = array(variable_data,float64).squeeze()
        else:
            line = file_obj.readline()
    
    for variable in ['L','strain']:
        try:
            if output.has_key( variable + '[0][0]' ):
                N = array(shape(output[variable + '[0][0]']))
                data = zeros(hstack([3,3,N]))
                for i in arange(3):
                    for j in arange(3):
                        data[i][j] = output.pop( variable + '[' + str(i) + '][' + str(j) + ']' )
                output[variable] = data
        except:
            print('"' + variable + '" is not contained.')
    return output

def factorization(ind_no):
    """Find prime numbers of N 
    INPUT = 
        ind_no : the number of indentation points
    OUTPUT =
        primes : prime factors of N
    """
    n = ind_no
    primes = []
    while n > 1:
        for i in arange(2,fix(n**0.5)+1):
            div = n/i
            if div-fix(div) == 0:
                primes.append(i)
                n = div
                break
        else:
            primes.append(n)
            n=1
    return primes

def get_sizes(primes,ind_no):
    """Returns all possible combination of discretization points in each direction 
    INPUT =
        primes : prime factors of N 
        ind_no : the number of indentation points
    OUTPUT =
        sizes : the list of all possible combinations of the number of discretization points in each direction
    """
    rows = []
    sizes = []
    for i in arange(2**len(primes)):
        bi = bin(i)[2:]
        row = 1
        for j in arange(len(bi)):
            if int(bi[-j-1]) != 0:
                row = row*primes[j]
        if row not in rows:
            rows.append(row)
            sizes.append([row,ind_no/row])
    sizes.sort()
    return sizes

def is_monotone(a):
    """returns the boolean value about monotonicity of vector a 
    INPUT =
        a : vector 
    OUTPUT =
        result : the answer about monotonicity of a; result = True means the vector a is monotone
    """
    if a[0] > a[-1]:
        a = flipud(a)
    result = True
    for i in arange(a.size-1):
        if a[i]>a[i+1]:
            result = False
    return result

def get_N(x0,y0,sizes):
    """Return the number of discretization points in each direction 
    INPUT =
        x0 : x coordinates of indentation points 
        y0 : y coordinates of indentation points
        sizes : the list of all admissible N  
    OUTPUT =
        N : the number of discretization points in each direction
    """
    if x0[0] > x0[-1]:
        x0 = x0[ ::-1]
    
    if y0[0] > y0[-1]:
        y0 = y0[ ::-1]

    x0_min = x0.min()
    x0_max = x0.max()
    y0_min = y0.min()
    y0_max = y0.max()
    test = zeros(2)
    minimum_of_tests = inf
    for size in sizes:
        x_rs = x0.reshape(size[0],size[1],order='F')
        y_rs = y0.reshape(size[0],size[1],order='F')
        #case 1
        x_app = transpose(tile(linspace(x0_min,x0_max,size[0]), (size[1],1)))
        y_app = tile(linspace(y0_min,y0_max,size[1]), (size[0],1))
        test[0] = linalg.norm(x_rs - x_app,2) + linalg.norm(y_rs - y_app,2)
        #case 2
        x_app = tile(linspace(x0_min,x0_max,size[1]), (size[0],1))
        y_app = transpose(tile(linspace(y0_min,y0_max,size[0]), (size[1],1)))
        test[1] = linalg.norm(x_rs - x_app,2) + linalg.norm(y_rs - y_app,2)
        if test.min() < minimum_of_tests:
            N = array(size)
            minimum_of_tests = test.min()
            
    N = N.flatten()
    return N

def read_results_CSM(file_name):
    """Read data from indentation file obtained using CSM  
    INPUT =
        file_name : the name of file to read from 
    OUTPUT =
        inputs : dict()
            N : the number of discretization points in each direction
            Y : the size of periodic unit cell (PUC)
            E : Young's modulus at indentation points
            nu : Poisson's ratio at indentation points
    """
    X = []
    Y = []
    E = []
    nu = []
    ind_no = 0
    variables = ['X','Y','EIT=',"Poisson's",'Indentation']
    for line in open(file_name,'r'):
        line_split = line.strip().split()
    #    print(line_split)
        if len(line_split) == 0:
            continue
        elif line_split[0] in variables:
            if line_split[0] == 'Indentation' and len(line_split)>2:
                if size(E)<ind_no: E.append('-1')
                if size(nu)<ind_no: nu.append('-1')
                ind_no = int(line_split[2].replace(',','.'))
                continue
            elif line_split[0] == 'EIT=':
                E.append(line_split[1].replace(',','.'))
                continue
            elif line_split[0] == "Poisson's":
                nu.append(line_split[2].replace(',','.'))
                continue
            elif line_split[0] == 'X':
                X.append(line_split[2].strip(':').replace(',','.'))
                continue
            elif line_split[0] == 'Y':
                Y.append(line_split[2].strip(':').replace(',','.'))        
    
    inputs = dict()
    x = array(X,float64)
    y = array(Y,float64)
    x0 = x - x.min()
    y0 = y - y.min()
        
    primes = factorization(ind_no)
    sizes = get_sizes(primes,ind_no)
    N = get_N(x0,y0,sizes)
    
    x0 = x0.reshape(N[0],N[1],order='F')
    y0 = y0.reshape(N[0],N[1],order='F')
    x = x.reshape(N[0],N[1],order='F')
    y = y.reshape(N[0],N[1],order='F')
    
    diff = zeros(2)
    diff[0] = max(abs(mean(x0[0]-x0[N[0]-1])),abs(mean(x0[:,0]-x0[:,N[1]-1])))
    diff[1] = max(abs(mean(y0[0]-y0[N[0]-1])),abs(mean(y0[:,0]-y0[:,N[1]-1])))
    
    E = array(E,float64)
    nu = array(nu,float64)
    h = diff/(N-1)
    Y = h*N
    E = E.reshape(N[0],N[1],order='F')
    nu = nu.reshape(N[0],N[1],order='F')
    inputs['E'] = E
    inputs['nu'] = nu
    inputs['N'] = N
    inputs['Y'] = Y
    inputs['h'] = h
    inputs['x'] = x
    inputs['y'] = y
    inputs['x0'] = x0
    inputs['y0'] = y0
    inputs['sizes'] = sizes
    return inputs
    
def read_results_Hys(file_name):
    """Read data from indentation file obtained using Hysitron  
    INPUT =
        file_name : the name of file to read from 
    OUTPUT =
        inputs : dict()
            N : the number of discretization points in each direction
            Y : the size of periodic unit cell (PUC)
            E : Young's modulus at indentation points
            nu : Poisson's ratio at indentation points
    """
    X = []
    Y = []
    Er = []
    ind_no = 0
    storage = 0
    variables = ['X','Y','EIT=',"Poisson's",'Indentation']
    no_of_line = 0
    for line in open(file_name,'r'):
        no_of_line += 1
        line_split = line.strip().split()
        if len(line_split) == 0:
            continue
        elif line_split[0] == 'Number':
            ind_no = int(line_split[-1])
            continue
        elif line_split[0] == 'File':
            storage = 1
            continue
        elif storage == 1:
            Er.append(line_split[-8].replace(',','.'))
            X.append(line_split[-3].strip(':').replace(',','.'))
            Y.append(line_split[-2].strip(':').replace(',','.'))
    
    inputs = dict()
    x = array(X,float64)
    y = array(Y,float64)
    x0 = x - x.min()
    y0 = y - y.min()
        
    primes = factorization(ind_no)
    sizes = get_sizes(primes,ind_no)
    N = get_N(x0,y0,sizes)
    
    x0 = x0.reshape(N[0],N[1],order='F')
    y0 = y0.reshape(N[0],N[1],order='F')
    x = x.reshape(N[0],N[1],order='F')
    y = y.reshape(N[0],N[1],order='F')
    
    diff = zeros(2)
    diff[0] = max(abs(mean(x0[0]-x0[N[0]-1])),abs(mean(x0[:,0]-x0[:,N[1]-1])))
    diff[1] = max(abs(mean(y0[0]-y0[N[0]-1])),abs(mean(y0[:,0]-y0[:,N[1]-1])))
    
    Er = array(Er,float64)
    h = diff/(N-1)
    Y = h*N
    Er = Er.reshape(N[0],N[1],order='F')
    inputs['E'] = Er
    inputs['N'] = N
    inputs['Y'] = Y
    inputs['h'] = h
    inputs['x'] = x
    inputs['y'] = y
    inputs['x0'] = x0
    inputs['y0'] = y0
    inputs['sizes'] = sizes
    return inputs

def read_analytical(file_name):
    """Reads data from file containing data for analytical homogenization
    Parameters:
    ----------
    INPUT:
     file_name : string (File)
        name of reading file
    OUTPUT:
     inputs : dictionary
         parameters for analytical homogenization
    """
    E = []
    Esd = []
    nu = []
    f = []
    no_phases = 0
    file_obj = open(file_name,'r')
    l_raw = file_obj.readline()
    while l_raw != "":
        l_strip = l_raw.strip()
        l_split = l_strip.split()
        l_splitR = l_strip.split('=')
        if len(l_split) == 0 or l_strip[0] == '#':
            l_raw = file_obj.readline()
            continue
        elif l_split[0] == 'Phase' and len(l_split)>1:
            if size(E)<no_phases: E.append('-1')
            if size(Esd)<no_phases: Esd.append('-1')
            if size(nu)<no_phases: nu.append('-1')
            if size(f)<no_phases: f.append('-1')
            no_phases += 1
            l_raw = file_obj.readline()
            continue
        elif l_splitR[0] == 'E':
            E.append(l_splitR[1].replace(',','.'))
            l_raw = file_obj.readline()
            continue
        elif l_splitR[0] == "E(s.d.)":
            Esd.append(l_splitR[1].replace(',','.'))
            l_raw = file_obj.readline()
            continue
        elif l_splitR[0] == 'nu':
            nu.append(l_splitR[1].replace(',','.'))
            l_raw = file_obj.readline()
            continue
        elif l_splitR[0] == 'f':
            f.append(l_splitR[1].replace(',','.'))
            l_raw = file_obj.readline()
            continue
        else:
            l_raw = file_obj.readline()
    
    inputs = dict()
    inputs['E'] = array(E,float64)
    inputs['Esd'] = array(Esd,float64)
    inputs['nu'] = array(nu,float64)
    inputs['f'] = array(f,float64)
    return inputs

#import matplotlib.pyplot as plt

def read_visco(file_name):
    """Reads data from indentation containing loading holding test 
    Parameters:
    ----------
    file_name : string (File)
        name of reading file
    Ei : Float
        Young's modulus of indenter
    nui : FLoat
        Poisson's ratio of indenter
    """
    #preallocation
    X = []; Y = []; 
    E = []; nu = []; M_0 = []; eta_M = []; G_V = []; eta_V = []
    t_L_save = []; t_H_save = []
                
    ind_no = 0
    variables = ['X','Y','EIT=',"Poisson's",'Indentation','Measured']
    file_obj = open(file_name,'r')
    l_raw = file_obj.readline()
    while l_raw != "":
        l_strip = l_raw.strip()
        l_split = l_strip.split()
        if len(l_split) == 0:
            l_raw = file_obj.readline()
            continue
        elif l_split[0] in variables:
            if l_split[0] == 'Indentation' and len(l_split)>2:
                if size(E)<ind_no: E.append('-1')
                if size(nu)<ind_no: nu.append('-1')
                ind_no = int(l_split[2].replace(',','.'))
            elif l_split[0] == 'EIT=':
                E.append(l_split[1].replace(',','.'))
            elif l_split[0] == "Poisson's":
                nu.append(l_split[2].replace(',','.'))
            elif l_split[0] == 'X':
                X.append(l_split[2].strip(':').replace(',','.'))
            elif l_split[0] == 'Y':
                Y.append(l_split[2].strip(':').replace(',','.'))
            elif l_split[0] == 'Measured':
                dummy = file_obj.readline().strip().split() #label
                l_split = file_obj.readline().strip().split()
                #preallocation
                Time = []
                Pd = []
                Fn = []
                while len(l_split) != 0:
                    Time.append(l_split[0].replace(',','.'))
                    Pd.append(l_split[1].replace(',','.'))
                    Fn.append(l_split[2].replace(',','.'))
                    l_split = file_obj.readline().strip().split()
                
                Time = array(Time,float64)
                Pd = array(Pd,float64)
                Fn = array(Fn,float64)
                l_raw = file_obj.readline()

                #calculation
                Fn_max = Fn.max()
                i1 = nonzero(Fn>Fn_max*0.2)[0][0]
                i2 = nonzero(Fn>Fn_max*0.9)[0][0]
                a1 = (Fn[i1]-Fn[i2])/(Time[i1]-Time[i2])
                t1 = -Fn[i1]/a1 + Time[i1] 
                t2 = (Fn_max-Fn[i1])/a1 + Time[i1]
                j1 = nonzero(Fn>Fn_max*0.2)[0][-1]
                j2 = nonzero(Fn>Fn_max*0.9)[0][-1]
                a2 = (Fn[j1]-Fn[j2])/(Time[j1]-Time[j2])
                t3 = (Fn_max-Fn[j2])/a2 + Time[j2]
                t4 = (-Fn[j2])/a2 + Time[j2]
                t_L = t2 - t1
                t_H = t3 - t2
                t_U = t4 - t3
                Fn_mmax = mean(Fn[nonzero((t2<Time)*(t3>Time))])
                a1 = Fn_mmax/t_L
                a2 = -Fn_mmax/t_U
                
                Pd_H = Pd[nonzero((t2<Time)*(t3>Time))]
                Time_H = Time[nonzero((t2<Time)*(t3>Time))]
                
                #fitting with KVM model
                E0 = float(E[-1])
                nu0 = float(nu[-1])
                M = E0/(1-nu0**2)

                #nonsimplified parameters
                T1 = lambda p: 6*p[0]*p[2]/(E0*(p[0]+p[2])+3*p[0]*p[1]+(E0**2*(p[0]+p[2])**2+6*E0*p[0]*p[1]*(p[0]-p[2])+9*p[0]**2*p[1]**2)**.5)
                T2 = lambda p: p[2]/p[1]
                T3 = lambda p: 6*p[0]*p[2]/(E0*(p[0]+p[2])+3*p[0]*p[1]-(E0**2*(p[0]+p[2])**2+6*E0*p[0]*p[1]*(p[0]-p[2])+9*p[0]**2*p[1]**2)**0.5)
                y_H = lambda p, t: (5-4*nu0)/(1-nu0**2) + M/p[1] - M*T2(p)/(p[1]*t_L)*exp(-t/T2(p))*(exp(t_L/T2(p))-1) + M/p[0]*(t-t_L/2) -(1-2*nu0)**2/(t_L*(1-nu0**2)*(1/T1(p)-1/T3(p)))*((T3(p)/T2(p)-1)*exp(-t/T3(p))*(exp(t_L/T3(p))-1)+(1-T1(p)/T2(p))*exp(-t/T1(p))*(t_L/T1(p)-1))  
                err_KVM = lambda p, t, y: (y_H(p,t)-y)**2
                
                #optimalization
                p0 = [30,.1,.1]
                resultsKVM = scipy.optimize.minpack.leastsq(err_KVM, p0, args=(Time_H,Pd_H),maxfev=1000)
                coef_KVM = resultsKVM[0]
                
                #allocation of 5 variables
                M_0.append(M)
                eta_M.append(coef_KVM[0])
                G_V.append(coef_KVM[1])
                eta_V.append(coef_KVM[2])
                t_L_save.append(t_L)
                t_H_save.append(t_H)
            else:
                pass
            l_raw = file_obj.readline()
            continue
        else:
            l_raw = file_obj.readline()
                
    inputs = dict()
    x = array(X,float64)
    y = array(Y,float64)
    x0 = x - x.min()
    y0 = y - y.min()
        
    primes = factorization(ind_no)
    sizes = get_sizes(primes,ind_no)
    N = get_N(x0,y0,sizes)
    
    x0 = x0.reshape(N[0],N[1],order='F')
    y0 = y0.reshape(N[0],N[1],order='F')
    x = x.reshape(N[0],N[1],order='F')
    y = y.reshape(N[0],N[1],order='F')
    
    M_0 = array(M_0,float64)
#    nu_0 = array(nu_0,float64)
    eta_M = array(eta_M,float64)
    G_V = array(G_V,float64)
    eta_V = array(eta_V,float64)
    t_L_save = array(t_L_save,float64)
    t_H_save = array(t_H_save,float64)
    
    #v-e parameters
    inputs['eta_M'] = eta_M.reshape(N[0],N[1],order='F')
    inputs['G_V'] = G_V.reshape(N[0],N[1],order='F')
    inputs['eta_V'] = eta_V.reshape(N[0],N[1],order='F')
    #auxiliary parameters
    inputs['t_L'] = t_L_save.reshape(N[0],N[1],order='F')
    inputs['t_H'] = t_H_save.reshape(N[0],N[1],order='F')
    
    diff = zeros(2)
    diff[0] = max(abs(mean(x0[0]-x0[N[0]-1])),abs(mean(x0[:,0]-x0[:,N[1]-1])))
    diff[1] = max(abs(mean(y0[0]-y0[N[0]-1])),abs(mean(y0[:,0]-y0[:,N[1]-1])))
    
    E = array(E,float64)
    nu = array(nu,float64)
    h = diff/(N-1)
    Y = h*N
    E = E.reshape(N[0],N[1],order='F')
    nu = nu.reshape(N[0],N[1],order='F')
    #other representative values
    inputs['E'] = E
    inputs['nu'] = nu
    inputs['N'] = N
    inputs['Y'] = Y
#    inputs['h'] = h
    return inputs        
